import { registerPlugin } from "@capacitor/core";
import type { SocketsPlugin } from "./definitions";
import mitt, { Emitter } from "mitt";

const Sockets = registerPlugin<SocketsPlugin>("Sockets");

type SocketEvents = {
	message: string;
	state: string;
};

class SocketManager {
	private static instance: SocketManager;
	private socketsMap: Map<string, Socket> = new Map();

	private constructor() {
		this.addGlobalListeners();
	}

	public static getInstance() {
		if (!SocketManager.instance) {
			SocketManager.instance = new SocketManager();
		}
		return SocketManager.instance;
	}

	public registerSocket(socket: Socket) {
		this.socketsMap.set(socket.id, socket);
	}

	public unregisterSocket(socket: Socket) {
		this.socketsMap.delete(socket.id);
	}

	private addGlobalListeners() {
		Sockets.addListener("state", (data) => {
			let socket = this.socketsMap.get(data.id);
			if (socket) {
				socket.didChangeState(data);
			}
		});

		Sockets.addListener("message", (data) => {
			let socket = this.socketsMap.get(data.id);
			if (socket) {
				socket.didReceiveMessage(data);
			}
		});
	}
}

export default class Socket {
	id: string;
	host: string;
	port: number;
	useTLS: boolean;
	acceptInvalidCertificates: boolean;

	private emitter: Emitter<SocketEvents>;

	constructor(config: { id: string; host: string; port: number; useTLS?: boolean; acceptInvalidCertificates?: boolean }) {
		console.log(config);

		this.id = config.id;
		this.host = config.host;
		this.port = config.port;
		this.useTLS = config.useTLS ?? false;
		this.acceptInvalidCertificates = config.acceptInvalidCertificates ?? false;

		this.emitter = mitt<SocketEvents>();

		this.create();

		SocketManager.getInstance().registerSocket(this);
	}

	on(event: keyof SocketEvents, handler: (data: any) => void) {
		this.emitter.on(event, handler);
	}

	create() {
		Sockets.create({
			id: this.id,
			host: this.host,
			port: this.port,
			useTLS: this.useTLS,
			acceptInvalidCertificates: this.acceptInvalidCertificates,
		});
	}

	connect() {
		Sockets.connect({
			id: this.id,
		});
	}

	disconnect() {
		Sockets.disconnect({
			id: this.id,
		});
	}

	close() {
		this.disconnect();
		SocketManager.getInstance().unregisterSocket(this);
	}

	send(message: string) {
		Sockets.send({
			id: this.id,
			message: message
		});
	}

	didChangeState(data: { state: string }) {
		this.emitter.emit("state", data.state);
	}

	didReceiveMessage(data: { message: string }) {
		this.emitter.emit("message", data.message);
	}
}
